/**
 * $Id$
 *
 * Copyright (C) shinichiro.h <hamaji@nii.ac.jp>
 *  http://shinh.skr.jp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "SDL_tex.h"

#include "mimetex.h"

SDL_Surface* SDLTex_Raster(const char* expr, int size,
                           SDL_Color fg, SDL_Color bg)
{
    raster* bp;
    SDL_Surface* s;
    int i, j;
    Uint32 fgp, bgp;
    Uint32* p;

    bp = make_raster(expr, size);

    s = SDL_CreateRGBSurface(SDL_SWSURFACE, bp->width, bp->height, 32,
                             0,0,0,0);

    bgp = SDL_MapRGB(s->format, bg.r, bg.g, bg.b);
    fgp = SDL_MapRGB(s->format, fg.r, fg.g, fg.b);
    SDL_FillRect(s, NULL, bgp);
    SDL_SetColorKey(s, SDL_SRCCOLORKEY, bgp);

    p = s->pixels;
    for (j = 0; j < bp->height; j++) {
        for (i = 0; i < bp->width; i++) {
            if (getpixel(bp, j, i)) *p = fgp;
            p++;
        }
    }

    return s;
}

SDL_Surface* SDLTex_RasterAA(const char* expr, int size,
                             SDL_Color fg, SDL_Color bg)
{
    raster* bp;
    SDL_Surface* s;
    int i, j;
    Uint32 fgp, bgp;
    Uint32* p;
    int dr, dg, db;

    intbyte* bytemap_raster;

    bp = make_raster(expr, size);

    bytemap_raster = (intbyte*)malloc(bp->width*bp->height);

    aalowpass(bp, bytemap_raster, 256);

    s = SDL_CreateRGBSurface(SDL_SWSURFACE, bp->width, bp->height, 32,
                             0,0,0,0);

    bgp = SDL_MapRGB(s->format, bg.r, bg.g, bg.b);
    fgp = SDL_MapRGB(s->format, fg.r, fg.g, fg.b);
    SDL_FillRect(s, NULL, bgp);
    SDL_SetColorKey(s, SDL_SRCCOLORKEY, bgp);

    dr = fg.r - bg.r;
    dg = fg.g - bg.g;
    db = fg.b - bg.b;

    p = s->pixels;
    for (j = 0; j < bp->height; j++) {
        for (i = 0; i < bp->width; i++) {
            int v = bytemap_raster[i+j*bp->width];
            if (v == 255) {
                *p = fgp;
            }
            else if (v != 0) {
                *p = SDL_MapRGB(
                    s->format,
                    ((dr*v)>>8)+bg.r, ((dg*v)>>8)+bg.g, ((db*v)>>8)+bg.b);
            }
            p++;
        }
    }

    free(bytemap_raster);

    return s;
}

